import argparse
import asyncio

from .logs import logger
from .scan import run_rustscan
from .validation import validate_cidr_or_ipv4
from bronzeburner.influx import InfluxDB


async def main(args: argparse.Namespace) -> None:
    """
    Invoked after parsing args. Checks InfluxDB state and initiates network scan.

    Args:
        args (argparse.Namespace): All the arguments passed to the command line

    Returns:
        int: Exit code from invoking rustscan

    Raises:
        SystemExit: When there is an error connecting to InfluxDB
    """
    if not await args.db.check_settings():
        logger.error("There was an error connecting to InfluxDB. Check your settings!")
        exit(1)
    return await run_rustscan(args)


def run() -> None:
    """
    This function creates and configures an ArgumentParser object for command line input,
    instantiates the InfluxDB class with the provided arguments, and asynchronously runs the main function.

    Command Line Arguments:
        -a, --address: IP address or CIDR range to scan. Required.
        -u, --url: InfluxDB server URL. Required.
        -o, --org: InfluxDB organization. Required.
        -b, --bucket: InfluxDB bucket. Required.
        -t, --token: InfluxDB token. Required.

    Raises:
        argparse.ArgumentError: If any of the required arguments are not provided
    """
    parser = argparse.ArgumentParser(description="A humble network scanner")
    parser.add_argument(
        "-a", "--address", help="IP address or CIDR range to scan", type=validate_cidr_or_ipv4, required=True
    )
    parser.add_argument("-u", "--url", help="InfluxDB server URL", type=str, required=True)
    parser.add_argument("-o", "--org", help="InfluxDB organization", required=True)
    parser.add_argument("-b", "--bucket", help="InfluxDB bucket", required=True)
    parser.add_argument("-t", "--token", help="InfluxDB token", required=True)
    args = parser.parse_args()

    args.db = InfluxDB(args.url, args.org, args.bucket, args.token)
    asyncio.run(main(args))


if __name__ == "__main__":
    run()
