import argparse
import asyncio
import re
from collections import namedtuple

from .influx import InfluxDB
from .logs import logger

PORTS_LINE_RE = re.compile(r"Host: (\d+\.\d+\.\d+\.\d+).*Ports: (.+)$")
PORT_ENTRY_RE = re.compile(r"(\d+)/([^/]*)/([^/]*)/([^/]*)/([^/]*)/([^/]*)/([^/]*)/")
PortEntry = namedtuple("PortEntry", "port state protocol owner service rpc_info version")


async def parse_output_line(db: InfluxDB, line: str) -> None:
    """
    This function parses a line of output from RustScan and stores port information in an InfluxDB database.

    Args:
        db (InfluxDB): The InfluxDB instance where the data will be stored.
        line (str): A single line of output from RustScan.

    Returns:
        None
    """
    if match := PORTS_LINE_RE.match(line):
        host_ip, port_findings = match.groups()
        port_entries = PORT_ENTRY_RE.findall(port_findings)
        parsed_ports = [PortEntry(*p) for p in port_entries]

        logger.warning(f"Found {len(port_entries)} ports for {host_ip}")
        for p in parsed_ports:
            logger.info(
                f"{host_ip}:{p.port} is {p.state} ({p.protocol}) with service {p.service}, version ({p.version}), and rpc_info ({p.rpc_info})"
            )

        if await db.insert(host_ip, parsed_ports):
            logger.info(f"Successfully wrote {len(parsed_ports)} ports to InfluxDB")
        else:
            logger.error(f"Failed to write {len(parsed_ports)} ports to InfluxDB")


async def run_rustscan(args: argparse.Namespace) -> int:
    """
    Runs rustscan with specified arguments and processes its output line by line using parse_output_line().

    Args:
        args (argparse.Namespace): Parsed command line arguments containing the address to scan

    Returns:
        int: The exit code of the rustscan process

    Raises:
        May raise various exceptions depending on the rustscan output and parsing process
    """
    rustscan_args = [
        "rustscan",
        "-t",
        "500",
        "-b",
        "1500",
        "--ulimit",
        "5500",
        "-a",
        str(args.address),
        "--",
        "-oG",
        "-",
    ]
    logger.info(f"Invoking rustscan: {' '.join(rustscan_args)}")

    process = await asyncio.create_subprocess_exec(*rustscan_args, stdout=asyncio.subprocess.PIPE)

    async for line in process.stdout:
        await parse_output_line(args.db, line.decode())

    return_code = await process.wait()
    if return_code != 0:
        logger.critical(f"rustscan exited with code {return_code}")

    return return_code
